@@ -1,21 +1,36 @@
-#! /usr/bin/env python3
-# Requires Python 3.6+
 # pylint: disable=invalid-name
 """Configuration of Sphinx documentation generator."""
 
-from pathlib import Path
+from __future__ import annotations
+
+import os
 import sys
+from pathlib import Path
+
+
+# -- Path setup --------------------------------------------------------------
+
+PROJECT_ROOT_DIR = Path(__file__).parents[1].resolve()
+IS_RELEASE_ON_RTD = (
+    os.getenv('READTHEDOCS', 'False') == 'True'
+    and os.environ['READTHEDOCS_VERSION_TYPE'] == 'tag'
+)
+if IS_RELEASE_ON_RTD:
+    tags: set[str]
+    # pylint: disable-next=used-before-assignment
+    tags.add('is_release')  # noqa: F821
 
 
 # Make in-tree extension importable in non-tox setups/envs, like RTD.
 # Refs:
 # https://github.com/readthedocs/readthedocs.org/issues/6311
 # https://github.com/readthedocs/readthedocs.org/issues/7182
-sys.path.insert(0, str(Path(__file__).parent.resolve()))
+sys.path.insert(0, str(PROJECT_ROOT_DIR))
 
 
 extensions = [
     'sphinx.ext.autodoc',
+    'sphinx.ext.autosectionlabel',  # autocreate section targets for refs
     'sphinx.ext.doctest',
     'sphinx.ext.extlinks',
     'sphinx.ext.intersphinx',
@@ -23,23 +38,21 @@ extensions = [
     'jaraco.packaging.sphinx',
     'sphinx_tabs.tabs',
     'sphinxcontrib.apidoc',
+    'sphinxcontrib.towncrier.ext',  # provides `.. towncrier-draft-entries::`
+    # In-tree extensions:
+    'spelling_stub_ext',  # auto-loads `sphinxcontrib.spelling` if installed
 ]
 
-# Conditional third-party extensions:
-try:
-    import sphinxcontrib.spelling as _sphinxcontrib_spelling
-except ImportError:
-    extensions.append('spelling_stub_ext')
-else:
-    del _sphinxcontrib_spelling  # noqa: WPS100
-    extensions.append('sphinxcontrib.spelling')
-
-# Tree-local extensions:
-extensions.append('scm_tag_titles_ext')
-
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ['_templates']
 
+# List of patterns, relative to source directory, that match files and
+# directories to ignore when looking for source files.
+# This pattern also affects html_static_path and html_extra_path.
+exclude_patterns = [
+    'changelog-fragments.d/**',  # Towncrier-managed change notes
+]
+
 master_doc = 'index'
 
 apidoc_excluded_paths = []
@@ -55,7 +68,8 @@ apidoc_toc_file = None
 
 spelling_ignore_acronyms = True
 spelling_ignore_importable_modules = True
-spelling_ignore_pypi_package_names = True
+# PyPI lookup because of https://github.com/sphinx-contrib/spelling/issues/227
+spelling_ignore_pypi_package_names = False
 spelling_ignore_python_builtins = True
 spelling_ignore_wiki_words = True
 spelling_show_suggestions = True
@@ -63,11 +77,6 @@ spelling_word_list_filename = [
     'spelling_wordlist.txt',
 ]
 
-scm_version_title_settings = {
-    'scm': 'git',
-    'date_format': '%d %b %Y',
-}
-
 github_url = 'https://github.com'
 github_repo_org = 'cherrypy'
 github_repo_name = 'cheroot'
@@ -97,46 +106,73 @@ intersphinx_mapping = {
     'trustme': ('https://trustme.readthedocs.io/en/latest/', None),
     'ddt': ('https://ddt.readthedocs.io/en/latest/', None),
     'pyopenssl': ('https://www.pyopenssl.org/en/latest/', None),
+    'towncrier': ('https://towncrier.rtfd.io/en/latest', None),
 }
 
 linkcheck_ignore = [
     r'http://localhost:\d+/',  # local URLs
     r'https://codecov\.io/gh/cherrypy/cheroot/branch/master/graph/badge\.svg',
     r'https://github\.com/cherrypy/cheroot/actions',  # 404 if no auth
-
     # Too many links to GitHub so they cause
     # "429 Client Error: too many requests for url"
     # Ref: https://github.com/sphinx-doc/sphinx/issues/7388
+    r'https://github\.com/cherrypy/cheroot/commit',
     r'https://github\.com/cherrypy/cheroot/issues',
     r'https://github\.com/cherrypy/cheroot/pull',
+    r'https://github\.com/cherrypy/cherrypy/commit',
     r'https://github\.com/cherrypy/cherrypy/issues',
     r'https://github\.com/cherrypy/cherrypy/pull',
-
     # Has an ephemeral anchor (line-range) but actual HTML has separate per-
     # line anchors.
     r'https://github\.com'
     r'/python/cpython/blob/c39b52f/Lib/poplib\.py#L297-L302',
     r'https://github\.com'
     r'/python/cpython/blob/c39b52f/Lib/poplib\.py#user-content-L297-L302',
+    r'^https://img\.shields\.io/matrix',  # these are rate-limited
+    r'^https://matrix\.to/#',  # these render fully on front-end from anchors
+    r'^https://stackoverflow\.com/',  # these generate HTTP 403 Forbidden
+    r'^https://forums\.sabnzbd\.org/',  # these generate HTTP 403 Forbidden
 ]
+linkcheck_anchors_ignore = [
+    r'^!',  # default
+    # ignore anchors that start with a '/', e.g. Wikipedia media files:
+    # https://en.wikipedia.org/wiki/Walrus#/media/File:Pacific_Walrus_-_Bull_(8247646168).jpg
+    r'\/.*',
+    r'issuecomment-\d+',  # GitHub comments
+]
+# FIXME: fossa timeout?
+linkcheck_timeout = 2  # default: 30
 linkcheck_workers = 25
 
+# -- Options for sphinx.ext.autosectionlabel extension -----------------------
+
+# Ref:
+# https://www.sphinx-doc.org/en/master/usage/extensions/autosectionlabel.html
+autosectionlabel_maxdepth = 1  # mitigate Towncrier nested subtitles collision
+
 nitpicky = True
 
 # NOTE: consider having a separate ignore file
 # Ref: https://stackoverflow.com/a/30624034/595220
 nitpick_ignore = [
-    ('py:data', 'SIGINT'),
     ('py:const', 'socket.SO_PEERCRED'),
     ('py:class', '_pyio.BufferedWriter'),
     ('py:class', '_pyio.BufferedReader'),
     ('py:class', 'unittest.case.TestCase'),
     ('py:meth', 'cheroot.connections.ConnectionManager.get_conn'),
-
     # Ref: https://github.com/pyca/pyopenssl/issues/1012
     ('py:class', 'pyopenssl:OpenSSL.SSL.Context'),
 ]
 
+# -- Options for towncrier_draft extension -----------------------------------
+
+# or: 'sphinx-version', 'sphinx-release'
+towncrier_draft_autoversion_mode = 'draft'
+towncrier_draft_include_empty = True
+towncrier_draft_working_directory = PROJECT_ROOT_DIR
+towncrier_draft_config_path = 'towncrier.toml'  # relative to cwd
+
+
 # Ref:
 # * https://github.com/djungelorm/sphinx-tabs/issues/26#issuecomment-422160463
 sphinx_tabs_valid_builders = ['linkcheck']  # prevent linkcheck warning
